"""
Tests for ingest_envoy.ingest
"""
import argparse
from unittest.mock import MagicMock, patch

import ingest_envoy.ingest as ingest

expected_settings = {
    "sdmId": "16B-069_sb32814386_1_001.57685.66193635417",
    "telescope": "EVLA",
    "workflowName": "std_calibration",
    "workflowRequestId": "2",
    "spl": "uid://evla/execblock/48ba4c9d-d7c7-4a8f-9803-1115cd52459b",
    "processingStart": "2021_07_06T21_50_48",
    "destinationDir": "/lustre/aoc/cluster/pipeline/docker/workspaces/spool/tmp576dknjp",
    "workflowDir": "tmp576dknjp",
    "staging_area": "/lustre/aoc/cluster/pipeline/docker/workspaces/staging",
    "storage_area": "/lustre/aoc/cluster/pipeline/docker/workspaces/storage",
}
expected_settings_image = {
    "sdmId": "brain_000.58099.67095825232",
    "telescope": "EVLA",
    "workflowName": "std_cms_imaging",
    "workflowRequestId": "12",
    "spl": "uid://evla/execblock/ec082e65-452d-4fec-ad88-f5b4af1f9e36",
    "processingStart": "2021_07_06T21_50_48",
    "destinationDir": "/lustre/aoc/cluster/pipeline/docker/workspaces/spool/tmprb1se376",
    "workflowDir": "tmprb1se376",
    "calSpl": "uid://evla/calibration/c78ccfd6-fe4e-43c6-a5c5-70e5bcfde16b",
    "staging_area": "/lustre/aoc/cluster/pipeline/docker/workspaces/staging",
    "storage_area": "/lustre/aoc/cluster/pipeline/docker/workspaces/storage",
}

args = argparse.Namespace
filename = "test/input_files/test-metadata.json"
filename2 = "test/input_files/test-image-metadata.json"


class TestIngest:
    def test_get_settings(self):
        settings = ingest._get_settings(filename, "calibration")
        assert settings["sdmId"] == expected_settings["sdmId"]
        assert settings["workflowName"] == expected_settings["workflowName"]
        assert settings["workflowRequestId"] == expected_settings["workflowRequestId"]
        assert settings["spl"] == expected_settings["spl"]
        assert settings["processingStart"] == expected_settings["processingStart"]
        assert settings["destinationDir"] == expected_settings["destinationDir"]
        assert settings["workflowDir"] == expected_settings["workflowDir"]
        assert settings["staging_area"] == expected_settings["staging_area"]
        assert settings["storage_area"] == expected_settings["storage_area"]

        settings2 = ingest._get_settings(filename2, "image")
        assert settings2["calSpl"] == expected_settings_image["calSpl"]

    def test_check_ingest_type(self):
        arg_type = "calibration"
        arg_type2 = "image"

        check = ingest.check_ingest_type(arg_type, expected_settings)
        assert check is True

        check2 = ingest.check_ingest_type(arg_type2, expected_settings)
        assert check2 is False

    def test_parser(self):
        parser = ingest.arg_parser()
        assert isinstance(parser, argparse.ArgumentParser)

    def test_main_calibration(self):
        args.calibration = [filename]

        with patch("argparse.ArgumentParser.parse_args", MagicMock(return_value=args)) as mock_args:
            with patch(
                "ingest_envoy.launchers.IngestCalibrationLauncher.launch_ingestion", return_value=0
            ) as cal_ingest:
                ingest.main()
                assert cal_ingest.call_count == 1
                assert cal_ingest.return_value == 0

        # reset for other testing
        args.calibration = None

    def test_main_image(self):
        args.image = [filename2]

        with patch("argparse.ArgumentParser.parse_args", MagicMock(return_value=args)) as mock_args:
            with patch(
                "ingest_envoy.launchers.IngestImageLauncher.launch_ingestion", return_value=0
            ) as img_ingest:
                ingest.main()
                assert img_ingest.call_count == 1
                assert img_ingest.return_value == 0

        # reset for other testing
        args.image = None
